Rem
Rem $Header: smxrw.sql 09-may-2002.18:57:51 tnallath Exp $
Rem
Rem smxrw.sql
Rem
Rem Copyright (c) 2001, 2002, Oracle Corporation.  All rights reserved.  
Rem
Rem    NAME
Rem      smxrw.sql 
Rem
Rem    DESCRIPTION
Rem      This is a demo script for DBMS_MVIEW.EXPLAIN_REWRITE().
Rem      We create a materialized view in SALES schema and
Rem      use EXPLAIN_REWRITE() to find out why query rewrite
Rem      failed for some queries. The output of EXPLAIN_REWRITE()
Rem      can go into a VARRAY or REWRITE_TABLE.
Rem
Rem
Rem    MODIFIED   (MM/DD/YY)
Rem    tnallath    05/09/02 - bug 2368367:.
Rem    mthiyaga    04/09/01 - Merged mthiyaga_xrw_demo
Rem    mthiyaga    04/06/01 - Created
Rem
Rem ===============================================================
Rem

CONNECT SH/SH;

SET ECHO ON;

DROP MATERIALIZED VIEW month_sales_mv;

CREATE MATERIALIZED VIEW month_sales_mv
  ENABLE QUERY REWRITE
  AS
  SELECT t.calendar_month_number,
         SUM(s.amount_sold) AS sum_dollars
  FROM sales s, times t
  WHERE s.time_id = t.time_id
  GROUP BY t.calendar_month_number;
  
REM Create REWRITE_TABLE for the output
  
DROP TABLE REWRITE_TABLE;

REM Create REWRITE_TABLE. Note the script to create the REWRITE_TABLE, utlxrw.sql, can 
REM be found in the /rdbms/admin directory.
  
@?/rdbms/admin/utlxrw.sql;
  

REM Enable query rewrite
ALTER SESSION SET QUERY_REWRITE_ENABLED = TRUE;

REM Stale tolerated mode
ALTER SESSION SET QUERY_REWRITE_INTEGRITY = STALE_TOLERATED;

DELETE FROM REWRITE_TABLE;

REM -----------------------------------------------------------------------------
REM Example 1: Output goes into REWRITE_TABLE
REM -----------------------------------------------------------------------------
  
SET SERVEROUTPUT ON;
DECLARE
   querytxt VARCHAR2(1000) := 'SELECT t.calendar_month_number, SUM(s.amount_sold) AS sum_dollars
   FROM sales s, times t
   WHERE s.time_id = t.time_id AND
         t.calendar_month_number = 5
   GROUP BY t.calendar_month_number';
BEGIN
 dbms_mview.Explain_Rewrite(querytxt, 'MONTH_SALES_MV');
END;
/

REM  Check the results in REWRITE_TABLE

SELECT * FROM REWRITE_TABLE;

REM -----------------------------------------------------------------------------
REM Example 2: Query doesn't have the join in MONTH_SALES_MV. 
REM -----------------------------------------------------------------------------


DELETE FROM REWRITE_TABLE;

EXECUTE dbms_mview.Explain_Rewrite('SELECT t.calendar_month_number, SUM(s.amount_sold) AS sum_dollars FROM sales s, times t  GROUP BY t.calendar_month_number', 'MONTH_SALES_MV');

SELECT * FROM REWRITE_TABLE;


REM -----------------------------------------------------------------------------
REM Example 3: In this example, we run explain_rewrite on two different queries. 
REM            Their outputs are distinguished by using two different statement 
REM            ids (query_1 and query_2).
REM -----------------------------------------------------------------------------

DELETE FROM REWRITE_TABLE;

SET SERVEROUTPUT ON;
DECLARE
   querytxt1 VARCHAR2(1000) := 'SELECT t.calendar_month_number, SUM(s.amount_sold) AS sum_dollars
   FROM sales s, times t
   WHERE s.time_id = t.time_id AND
         t.calendar_month_number = 5
     GROUP BY t.calendar_month_number';
   querytxt2 VARCHAR2(1000) := 'SELECT t.calendar_month_number, SUM(s.amount_sold) AS sum_dollars
   FROM sales s, times t
   WHERE s.time_id = t.time_id AND
         t.calendar_month_number = 10
     GROUP BY t.calendar_month_number';
  
BEGIN
 dbms_mview.Explain_Rewrite(querytxt1, 'MONTH_SALES_MV', 'query_1');
 dbms_mview.Explain_Rewrite(querytxt2, 'MONTH_SALES_MV', 'query_2');
END;
/

SELECT statement_id, message FROM REWRITE_TABLE;


REM -----------------------------------------------------------------------------
REM Test 4:  Query txt is the only argument passed to EXPLAIN_REWRITE. Output
REM          goes to REWRITE_TABLE. Note that since there is no materialized view
REM          specified in the argument list, EXPLAIN_REWRITE will output
REM          messages related to all the MV's considered by query rewrite.
REM -----------------------------------------------------------------------------


DELETE FROM REWRITE_TABLE;

SET SERVEROUTPUT ON;
DECLARE
   querytxt VARCHAR2(1000) := 'SELECT t.calendar_month_number, SUM(s.amount_sold) AS sum_dollars
   FROM sales s, times t
   WHERE t.calendar_month_number = 5
   GROUP BY t.calendar_month_number';
BEGIN
 dbms_mview.Explain_Rewrite(querytxt);
END;
/

-- Check the results in REWRITE_TABLE

SELECT * FROM REWRITE_TABLE;



REM -----------------------------------------------------------------------------
REM Test 5: Output goes into a VARRAY. Note that we need to turn on SERVEROUTPUT
REM         in order to see the output below.
REM -----------------------------------------------------------------------------

SET SERVEROUTPUT ON;
DECLARE
   Rewrite_Array SYS.RewriteArrayType := SYS.RewriteArrayType();
   querytxt VARCHAR2(1000) := 'SELECT t.calendar_month_number, SUM(s.amount_sold) AS sum_dollars
   FROM sales s, times t
   WHERE s.time_id = t.time_id AND
         t.calendar_month_number = 5
   GROUP BY t.calendar_month_number';
   no_of_msgs NUMBER;
   i NUMBER;
BEGIN
 dbms_mview.Explain_Rewrite(querytxt, 'MONTH_SALES_MV',  Rewrite_Array);
   no_of_msgs := rewrite_array.count;
   FOR i IN 1..no_of_msgs
   LOOP
      DBMS_OUTPUT.PUT_LINE('>> MV_NAME  : ' || Rewrite_Array(i).mv_name);
      DBMS_OUTPUT.PUT_LINE('>> BEFORE VM: ' || Rewrite_Array(i).pass);
      DBMS_OUTPUT.PUT_LINE('>> QUERY    : ' || Rewrite_Array(i).query_text);
      DBMS_OUTPUT.PUT_LINE('>> MESSAGE  : ' || Rewrite_Array(i).message);
   END LOOP;
END;
/



REM -----------------------------------------------------------------------------
REM Test 6: Query rewrite is disabled
REM -----------------------------------------------------------------------------

-- Disable query rewrite and try the query
ALTER SESSION SET QUERY_REWRITE_ENABLED = FALSE;

SET SERVEROUTPUT ON;
DECLARE
   Rewrite_Array SYS.RewriteArrayType := SYS.RewriteArrayType();
   querytxt VARCHAR2(1000) := 'SELECT t.calendar_month_number, SUM(s.amount_sold) AS sum_dollars
   FROM sales s, times t
   WHERE s.time_id = t.time_id AND
         t.calendar_month_number = 5
   GROUP BY t.calendar_month_number';
   no_of_msgs NUMBER;
   i NUMBER;
BEGIN
 dbms_mview.Explain_Rewrite(querytxt, 'MONTH_SALES_MV',  Rewrite_Array);
   no_of_msgs := rewrite_array.count;
   FOR i IN 1..no_of_msgs
   LOOP
      DBMS_OUTPUT.PUT_LINE('>> MV_NAME  : ' || Rewrite_Array(i).mv_name);
      DBMS_OUTPUT.PUT_LINE('>> BEFORE VM: ' || Rewrite_Array(i).pass);
      DBMS_OUTPUT.PUT_LINE('>> QUERY    : ' || Rewrite_Array(i).query_text);
      DBMS_OUTPUT.PUT_LINE('>> MESSAGE  : ' || Rewrite_Array(i).message);
   END LOOP;
END;
/

REM -----------------------------------------------------------------------------
REM Test 7: Query rewrite is enabled
REM -----------------------------------------------------------------------------

REM  Enable query rewrite and try the query again
ALTER SESSION SET QUERY_REWRITE_ENABLED = TRUE;

SET SERVEROUTPUT ON;
DECLARE
   Rewrite_Array SYS.RewriteArrayType := SYS.RewriteArrayType();
   querytxt VARCHAR2(1000) := 'SELECT t.calendar_month_number, SUM(s.amount_sold) AS sum_dollars
   FROM sales s, times t
   WHERE s.time_id = t.time_id AND
         t.calendar_month_number = 5
   GROUP BY t.calendar_month_number';
   no_of_msgs NUMBER;
   i NUMBER;
BEGIN
 dbms_mview.Explain_Rewrite(querytxt, 'MONTH_SALES_MV',  Rewrite_Array);
   no_of_msgs := rewrite_array.count;
   FOR i IN 1..no_of_msgs
   LOOP
      DBMS_OUTPUT.PUT_LINE('>> MV_NAME  : ' || Rewrite_Array(i).mv_name);
      DBMS_OUTPUT.PUT_LINE('>> BEFORE VM: ' || Rewrite_Array(i).pass);
      DBMS_OUTPUT.PUT_LINE('>> QUERY    : ' || Rewrite_Array(i).query_text);
      DBMS_OUTPUT.PUT_LINE('>> MESSAGE  : ' || Rewrite_Array(i).message);
   END LOOP;
END;
/

REM -----------------------------------------------------------------------------
REM Clean up
REM -----------------------------------------------------------------------------
  
DROP MATERIALIZED VIEW MONTH_SALES_MV;
DISCONNECT;


